/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.ImageDiffer;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.gfxtest.framework.Log;
import org.gfxtest.ImageDiffer.ResultWriters.HtmlStructureWriter;
import org.gfxtest.ImageDiffer.ResultWriters.HtmlWriter;
import org.gfxtest.ImageDiffer.ResultWriters.ResultWriter;
import org.gfxtest.ImageDiffer.ResultWriters.XmlWriter;

public class Main
{
    private static final String VERSION = "0.1";
    private int errorsCounter = 0;
    private int differentImageCounter = 0;
    private int sameImageCounter = 0;
    private static Log log = new Log(Main.class.getName(), true);

    public void run(String[] args) throws Exception
    {
        Configuration configuration = new Configuration(args);
        List<ResultWriter> resultWriters = new ArrayList<ResultWriter>();

        if (configuration.isXmlResultCreationEnabled())
        {
            resultWriters.add(new XmlWriter(configuration.getOutputDirectory()));
        }
        if (configuration.isHtmlResultCreationEnabled())
        {
            resultWriters.add(new HtmlWriter(configuration.getOutputDirectory()));
        }

        try
        {
            String[] files = configuration.getFirstSourceDirectory().list();
            Arrays.sort(files);
            for (String imageFileName : files)
            {
                testImage(configuration, imageFileName, resultWriters);
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            for (ResultWriter resultWriter : resultWriters)
            {
                resultWriter.printFooter();
                resultWriter.close();
            }
        }

        log.logSet("same images", Integer.valueOf(this.sameImageCounter));
        log.logSet("different images", Integer.valueOf(this.differentImageCounter));
        log.logSet("errors", Integer.valueOf(this.errorsCounter));
    }

    private static void printHelp()
    {
        System.out.println(
                "Welcome to Lupic, the yet another image comparator\n\n" +
                "(c) 2010, 2011, 2012 Pavel Tisnovsky, Red Hat Inc.\n\n" +
                " Usage:\n " +
                "java -jar lupic.jar -s1=./samples -s2=./output -m=./masks -b=./bitmap-masks -o=./output --html --html-results --xml-results --diff-images --struct-diff-images\n" +
                "\n" +
                " Command line options:\n" +
                "    -h,  --help                prints this help\n" +
                "    -s1, --source1             directory containing samples\n" +
                "    -s2, --source2             directory containing generated images\n" +
                "    -o,  --output              output directory\n" +
                "    -t,  --html                enable generating HTML file for each comparison output\n" +
                "    -r,  --html-results        enable generating HTML containing test report\n" +
                "    -x,  --xml-results         enable generation XML containing test report\n" +
                "    -d,  --diff-images         whether to create diff-images\n" +
                "    -s,  --struct-diff-images  whether to create images containing structures of differences\n" +
                "");
    }

    private static void printNoParametersGiven()
    {
        System.out.println("Use -h or --help option to show help.");
    }

    private static void printVersion()
    {
        System.out.println("Lupic version " + VERSION);
    }

    private static boolean argsContainHelp(String[] args)
    {
        return argsContainString(args, new String[] {"-h", "-help", "--help", "-?", "--?"});
    }

    private static boolean argsContainsVersion(String[] args)
    {
        return argsContainString(args, new String[] {"-ver", "-version", "--version"});
    }

    private static boolean argsContainString(String[] args, String[] options)
    {
        for (String arg : args)
        {
            for (String option : options)
            {
                if (arg.equals(option))
                {
                    return true;
                }
            }
        }
        return false;
    }

    private void testImage(Configuration configuration, String imageFileName, List<ResultWriter> resultWriters)
    {
        log.logBegin(String.format("testing image %s", imageFileName));
        try
        {
            BufferedImage[] sourceImages = ImageUtils.readSourceImages(configuration.getFirstSourceDirectory(), configuration.getSecondSourceDirectory(), imageFileName);
            ComparisonResult comparisonResult = new ImageComparator().diffImages(imageFileName, sourceImages, configuration);
            if (!comparisonResult.isEqualsImages())
            {
                this.differentImageCounter++;
                log.logSet("DIFFER", imageFileName);
            }
            else
            {
                this.sameImageCounter++;
                log.logSet("SAME", imageFileName);
            }
            if (configuration.isDiffImagesCreationEnabled())
            {
                writeDiffImage(configuration, imageFileName, comparisonResult.getDiffImage());
            }
            if (configuration.isStructDiffImagesCreationEnabled())
            {
                writeStructDiffImage(configuration, imageFileName, sourceImages, comparisonResult.getDiffImage());
            }
            for (ResultWriter resultWriter : resultWriters)
            {
                resultWriter.printResultForOneImage(configuration, imageFileName, sourceImages, comparisonResult);
            }
            if (configuration.isHtmlFilesCreationEnabled())
            {
                String dirName = imageFileName.substring(0, imageFileName.lastIndexOf('.'));
                HtmlStructureWriter.createAndFillResultDirectory(configuration, sourceImages, comparisonResult, dirName);
            }
        }
        catch (Exception e)
        {
            this.errorsCounter++;
            log.logError(e.getMessage());
            e.printStackTrace(System.err);
        }
        finally
        {
            log.logEnd(String.format("testing image %s", imageFileName));
        }
    }

    private void writeStructDiffImage(Configuration configuration, String imageFileName, BufferedImage[] sourceImages,
            BufferedImage diffImage) throws IOException
    {
        String dirName = imageFileName.substring(0, imageFileName.lastIndexOf('.'));
        File newDir = new File(configuration.getOutputDirectory(), dirName);
        log.logProcess("creating directory %s", newDir.getAbsolutePath());
        if (!newDir.mkdir()) {
            log.logError("error creating directory %s", newDir.getAbsolutePath());
        }
        log.logProcess("writing source and diff images to directory %s", dirName);
        ImageUtils.writeImage(sourceImages[0], newDir, "source1.png");
        ImageUtils.writeImage(sourceImages[1], newDir, "source2.png");
        ImageUtils.writeImage(diffImage, newDir, "diff.png");
    }

    private void writeDiffImage(Configuration configuration, String imageFileName, BufferedImage diffImage)
            throws IOException
    {
        ImageUtils.writeImage(diffImage, configuration.getOutputDirectory(), imageFileName);
        log.logProcess("writing diff image %s", imageFileName);
    }

    /**
     * Entry point to this tool.
     *
     * @param args
     */
    public static void main(String[] args)
    {
        if (args.length == 0)
        {
            Main.printNoParametersGiven();
            return;
        }
        if (Main.argsContainHelp(args))
        {
            Main.printHelp();
            return;
        }
        if (Main.argsContainsVersion(args))
        {
            Main.printVersion();
            return;
        }
        long t1 = System.currentTimeMillis();
        try
        {
            log.logBegin(String.format("started at %s", new Date().toString()));
            new Main().run(args);
        }
        catch (Exception e)
        {
            System.err.println(e.getMessage());
            e.printStackTrace();
        }
        finally
        {
            long t2 = System.currentTimeMillis();
            log.logSet("run duration", Long.valueOf((t2 - t1) / 1000));
            log.logEnd(String.format("finished at %s", new Date().toString()));
        }
    }

}
